# coding=utf-8


from grecx.datasets.light_gcn_dataset import LightGCNYelpDataset, LightGCNGowallaDataset, LightGCNAmazonbookDataset
from grecx.datasets.diff_net_dataset import DiffNetYelp, DiffNetFlickr


from . import light_gcn_dataset
from . import diff_net_dataset